#  Linux 101
## Getting Started
(CC0) Starbase17.space 2021

### ToC
1 Linux and the Linux Philosophy
2 Basic Linux Architecture
3 Using the CLI

A1 Markdown Supplement
A2 Solutions and Explanations

### Linux and the Linux Philosophy
In this section, we are going to discuss Linux as a concept, an operating
system, and a philosophy. 
We will touch first on the history of the operating system and discuss 
various people important in the building the foundations of the linux 
world. Finally, we will talk a bit about the philosophy of linux, and how
it can be applied.  

#### Linux History
The history of linux really starts in the 1960s. At this time, AT&T and 
Bell labs, among many others, were working on operating systems for 
large mainframes. AT&T/Bell left this project (Multics) and began 
working on an operating system of their own with developers Ken Thompson 
and Dennis Ritchie. This operating system was released as AT&T Unix. 

Meanwhile, UC Berkley was working on the Berkley Software Distribution
(BSD) This was going to be an open source operating system from the 
Berkley Computer Systems Research Group. As this incorporated code from
AT&T Unix, AT&T eventually sued to hinder the development of BSD. It 
worked. 

In the early 80s, Richard Stallman gave us the GNU's not Unix (GNU)
project. This project was (is) dedicated to creatind a free Unix-like
OS. The GNU Kernel, however, is still incomplete, which leads to gnu/linux
systems. These are GNU philosophy and licensed software built on top of a
linux kernel. 

Skipping ahead to the early 1990s, Linus Torvalds put together a project
to supplant MINIX for a free operating system. In August of 1991, he 
posted to a MINIX group on usenet that he had ported `bash` and `gcc` to 
his new Linux system. It was non-portable and only supported a single kind
of hard disk and file system. 

At this point, things were well underway, but in 1992 the X Window System 
was ported to the Linux kernel and things really took off. Over the years
this lead to the development of more and more variants of Linux. 

#### The Linux Movement
In the early 90s we started seeing forks of Linux. There was the Debian 
operating system, the Red Hat operating system, and SUSE. Red Hat and
SUSE are both commercial operating systems, providing the operating 
system code for free but offering services on top of the OS such as 
support and patch management. It also started to see adoption by 
large companies who would directly support the linux movement. These
companies included IBM, Compaq, Dell, and others. 

In the ecosystem today there are hundreds to thousands of linux 
distributions, all with different foci and goals. There are distributions
geard towards privacy, various religions, even My Little Pony. 

The Linux movement is also guided by the Linux philosophy, which we will
discuss in detail below. In short, it is a free as in liberty philosophy.
With that philosophy also comes a wealth of other free and open source 
software, from productivity and development apps, to games and media
players.  

#### The Linux Philosophy

The Linux Philosophy is given to us pretty succinctly in Mike Gancarz's
book __Linux and the Unix Philosophy__. In this work he lists 9 major
tenants of the Linux Philosophy. 

```
1. Small is beautiful. Small things have tremendous advantages over their larger
counterparts. Among these is the ability to combine with other small things in 
unique and useful ways, ways often unforeseen by the original designer.

2. Make each program do one thing well. By focusing on a single task, a program 
can eliminate much extraneous code that often results in excess overhead, 
unnecessary complexity, and a lack of flexibility.

3. Build a prototype as soon as possible. Most people would agree that 
prototyping is a valuable element of any project. But whereas prototyping is 
only a small part of the design phase under other methodologies, under Unix it 
is the principal vehicle for generating an effective design.

4. Choose portability over efficiency. When Unix broke new ground as the first 
portable operating system of any significance, it was big news. Today 
portability is taken for granted as a necessity in any modern software design, 
an example of a tenet that has gained wide acceptance on other systems besides 
Unix.

5. Store data in flat text files. The choice between portability and efficiency 
addresses the value of portable code. Portable data is at least as important 
as-if not more important than-portable code. Portable data is the 
often-neglected part of the portability formula.

6. Use software leverage to your advantage. Many programmers have only a 
superficial understanding of the importance of reusable code modules. Code reuse
 helps one take advantageof software leverage, a powerful concept that some Unix
  developers use to create numerous applications in a comparatively short time.

7. Use shell scripts to increase leverage and portability. Shell scripts are 
double-edged swords for enhancing both software leverage and portability in a 
design. Whenever possible, writing a script instead of a complete C program is 
the way to go.

8. Avoid captive user interfaces. Some commands have user interfaces known to 
Unix developers as "captive" user interfaces. These prevent the user from 
running other commands while the command is in use, effectively making the user 
a captive to the system for the duration of the command. In a graphical user 
interface world, such interfaces would be called "modal."

9. Make every program a filter. The fundamental nature of all software programs 
is that they only modify data; they do not create it. Therefore, they should be 
written to perform as filters since they are filters```

(The 9 tenants, quoted from Mike Gancarz's book __Linux and the Unix__
__Philosophy__)[1]

Effectively, this philosophy of design is what brings us to having 
systems that are perfectly usable as a command line interface, with a 
significant amount of functionality. In truth, the Linux GUI options 
are tacked on top of the shell. 

Also baked into the Linux Movement is the notion of FLOSS. This 
is the ideology of Free (Libre) Open Source Software. A lot of 
software out there is FOSS, but comes with heavy restrictions on 
use, ability to derive from, ability to redistribute or otherwise. FLOSS
advocates tend to believe that data and information should flow freely, 
and that software is no different. This means that many things you find in 
the libre community will have little-to-no limits on use, and when they 
do the limits are sensible. Most commonly the limits you will see are: 

* Attribution - You must attribute the source to the creator
* Share-Alike - You must share the code and your work with a similar and 
  compatible license
* Non-Commercial - You may not use this work to make profit

There is often comparison to GPL-compatible licenses. These are all 
licenses that are compatible with the GNU General Public License, which 
is a fairly open license for open source software. It does has 
restrictions on sublicensing and liability.[2]

For instance, the course you are currently reviewing is released under 
the CC0 license. Meaning this worked has 0 license. Meaning it can be 
used by anyone and for any reason. 

#### Endnotes

[1] Gancarz, M. (2003). Linux and the Unix philosophy. Digital Press.
[2] gnugpl.org

=========================================================================
### Basic Linux Architecture

This section is going to be a __very__ high-level primer on the 
architecture of the operating system, explained in the ways you need to 
know for now. Most of these concepts will be dug into pretty deeply 
over the course of the course. 

#### The Linux Kernel

The Linux system exists in a number of abstract spaces. We will cover 
these in more detail as we go through the course, but what you need to 
know now is that the system portion that deals directly with the hardware
isn't directly used by the user, and the portion that directly deals with 
the user isn't directly used by the hardware. All of the communication
between the user and the hardware and vice-versa is handled by the GNU
C Library (glibc).  

```
+--------+  glibc   +------+
| Kernel | <------> | user |
+--------+          +------+
```

We have talked a little bit about the linux kernel, briefly, before now. 
The linux kernel is the core of the operating system. It is a series of 
software modules that allow for manipulation of a computers hardware. 
Specifically, it consists of files and code that allows the operating
system to manipulate the hardware based on the instructions of the user. 

This kernel allows the operating system, for instance, to interact with 
the disk hardware. The disk itself holds the filesystem that stores the
files that you interact with. The kernel can then interpret the 
filesystem's information to provide the user knowledge about what is 
stored on the disk. 

#### The Filesystem
An important aspect of linux is the filesystem. The filesystem is the 
way data is encoded on the disk, and stored for later use. At the base
level, disks are either electronic or magenetic media that store the 
data in a binary-encoded fashion. This means that the information is
stored in the form of 1s and 0s on the disk that the computer can use
to give you the letters you see on the screen. 

The filesystem defines what those 1s and 0s mean, and how the computer
can interpret them in a meaningful way for the user. The filesystem 
stores these as, generally, files and directories. Directories are
lists of files that have been grouped together. Files are objects that
contain useful data and information. For instance, your home directory
is a listing of the files you have stored on the system, and the default
files provided to you. The individual files have text and binary data
that can be read. If you want to understand a little bit more about 
how this works, try using the `vi` command on a directory. You can see
the actual text that forms the directory. In the linux operating 
system most things are represented as files. This allows for us to 
use the operating system in very useful ways, as we will explore. 

#### Common Directories
In order to further make navigating the filesystem easier, the linux 
operating system uses a hierarchy standard for its filesystem. The
aptly named Linux Filesystem Hierarchy Standard.[1] This defines
what all of the directories and special use files in the filesystem
are used for and what they contain. (Feel free to read the FHS if you
choose) The common directories you need to know at this point are:

* /home/
 * The user folders on the system
* /
 * The root folder of the filesystem

As we work through the course, more and more directories will become
important.

#### Endnotes
[1] https://refspecs.linuxfoundation.org/FHS_3.0/fhs/index.html

=========================================================================
### Using the CLI
Now we know some basics about the linux operating system, let's look
at some simple CLI commands to get you started. It is recommended that
you keep a terminal open to try some of these commands. 

One of the first things to do when you log into an operating system, 
especially if you need to figure out what you're doing, is list the 
contents of the directory you are in. The `ls` command is used to 
list the contents of the directory. It has a number of *switches* that
can be set. Switches are parameters that can be passed to many command
line programs that affect their operation. 

The `ls` program has a few very common switches: 
* The `-l` switch can be used to provide a long listing of the files with 
additional permissions information, file size, and more. 
* The `-a` switch is used to list all files in the directory, even hidden
files. 
* The `-h` switch can be used to make listed file sizes human readable. 

The `pwd` command can be used to list your present working directory. 
This can be useful when you are navigating deep, labyrinthine application
directories. Especially when you don't recognize the files in the
directory you are in. 

If you find that you're in the wrong directory, `cd` can be used to 
change directories. The usage is `cd /new/directory/path` where "new/
directory/path" is replaced with the path to the directory you want to
be in. For instance, if you want to look at the apache2 logs you would
navigate to "/etc/logs/apache2".

Once you find yourself in a directory, you can use `cat` and `pico` to
look at text files. `cat` will display the contents of any file passed
to it as an argument to standard out. An argument is any text passed to 
a file, and standard out (STDOUT) is the text you see in the terminal 
when an application is run. The `cat` command is short for con`cat`enate. 
It's design purpose is to merge the output of two files to standard out. 
This can be very useful. `pico` is a small text editor that can be used
to review and edit text files. It's use is fairly intuitive. There are
hints on what commands can be used to do what at the bottom of the screen
within `pico`. *Note: the `^` character is a `CTRL` key*

If you don't want to open `pico` and a file is too long to see the whole
thing when you `cat` it to the screen, you can use `less`. `less` is a 
paging utility. This means it will display the file, from top to bottom
in pages that can be displayed in your terminal. The arrow keys, and 
`pgup` and `pgdn` will allow you to navigate the file. Press the `q` key
to quit. To use `less` you will need to "pipe" the output of the `cat`
command to the `less` command. "Piping" is a method of *redirecting* 
STDOUT of the `cat` to the STDIN (standard in, which is the input to
an application in this context) of `less`. 

Now, if at any point you need to know more about one of these commands, 
you can use the `man` command to view the manual for other commands. 
The `man` command is used like `cd` or `pico` where you pass the filename
as an argument. So to read the manual for `cd` you would run `man cd`.  

At this point, you truly have enough knowledge and skill with the command
line that you could learn the rest on your own. We do intend to expand 
this series with more lessons on increasingly advanced concepts. Keep
an eye on `bb` for more info on module and course releases!

=========================================================================
### Markdown Supplement

*Note: The supplement is not necessary reading, but useful for expanding*
*your information and skills*

#### What is Markdown?
Markdown is a common editing markup language (markdown language?) for
stylizing text. Since you're probably reading this in `pico` if you 
followed the instructions to get here, it is what causes some parts to 
be different colors to highlight their position. 

For instance, the `#` character is used to designate different levels of
heading. Some common markdown characters are: 

* `*` used to make bulleted lists
* `#` used to indicate headers
* ` is used to start and close code segments
* ``` is used to start and close multi-line blocks of code or quotes
* `*` when wrapped around a word can be used to *bold* it
* `__` when wrapped around a word will make it __italic__.

When working on your document, try to keep your file at no more than 74
columns wide (80 is the max). This is a standard convention, but beyond
that, use the styling in your own way. It's about using the formatting
to convey your message correctly. 

### Solutions and Explanations

This lab doesn't really have any problems to solve. You're just
going to be writing a markdown file. 
